"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRepository = void 0;
const knex_utils_1 = require("knex-utils");
const NonUniqueResultError_1 = require("./NonUniqueResultError");
const NoEntityExistsError_1 = require("./NoEntityExistsError");
const dbSupportHelper_1 = require("./dbSupportHelper");
const objectUtils_1 = require("knex-utils/dist/lib/objectUtils");
class AbstractRepository {
    constructor(knex, config) {
        var _a, _b, _c, _d, _e;
        this.knex = knex;
        this.tableName = config.tableName;
        this.idColumn = config.idColumn;
        this.defaultOrderBy = config.defaultOrderBy;
        this.columnsToFetch = config.columnsToFetch || ['*'];
        this.columnsToFetchList = (_b = (_a = config.columnsToFetch) !== null && _a !== void 0 ? _a : config.columnsToFetch) !== null && _b !== void 0 ? _b : ['*'];
        this.columnsToFetchDetails = (_d = (_c = config.columnsToFetch) !== null && _c !== void 0 ? _c : config.columnsToFetch) !== null && _d !== void 0 ? _d : ['*'];
        this.columnsForCreate = config.columnsForCreate || undefined;
        this.columnsForFilters = config.columnsForFilters || undefined;
        this.columnsForUpdate = config.columnsForUpdate || undefined;
        this.columnsForCreateSet = this.columnsForCreate ? new Set(this.columnsForCreate) : undefined;
        this.columnsForFiltersSet = this.columnsForFilters ? new Set(this.columnsForFilters) : undefined;
        this.columnsForUpdateSet = this.columnsForUpdate ? new Set(this.columnsForUpdate) : undefined;
        this.throwOnInvalidColumns =
            (_e = (config.throwOnInvalidColumns &&
                (this.columnsForCreateSet !== undefined ||
                    this.columnsForFiltersSet !== undefined ||
                    this.columnsForUpdateSet !== undefined))) !== null && _e !== void 0 ? _e : false;
    }
    pickWithoutUndefined(source, propNames, propSet) {
        if (this.throwOnInvalidColumns && propSet) {
            (0, objectUtils_1.validateOnlyWhitelistedFieldsSet)(source, propSet);
        }
        return (0, knex_utils_1.pickWithoutUndefined)(source, propNames);
    }
    async create(newEntityRow, transactionProvider) {
        const insertRow = this.columnsForCreate
            ? this.pickWithoutUndefined(newEntityRow, this.columnsForCreate, this.columnsForCreateSet)
            : (0, knex_utils_1.copyWithoutUndefined)(newEntityRow);
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const insertedRows = await queryBuilder(this.tableName)
            .insert(insertRow)
            .returning(this.columnsToFetch);
        if (!(0, dbSupportHelper_1.doesSupportReturning)(this.knex)) {
            const insertedRow = await this.getById(insertedRows[0], transactionProvider);
            return insertedRow;
        }
        return insertedRows[0];
    }
    async createBulk(newEntityRows, transactionProvider, params = { chunkSize: 1000 }) {
        const insertRows = newEntityRows.map((newEntityRow) => this.columnsForCreate
            ? this.pickWithoutUndefined(newEntityRow, this.columnsForCreate, this.columnsForCreateSet)
            : (0, knex_utils_1.copyWithoutUndefined)(newEntityRow));
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const chunks = (0, knex_utils_1.chunk)(insertRows, params.chunkSize);
        const insertedRows = [];
        for (const rows of chunks) {
            insertedRows.push(...(await queryBuilder(this.tableName).insert(rows).returning(this.columnsToFetch)));
        }
        return insertedRows;
    }
    async createBulkNoReturning(newEntityRows, transactionProvider, params = { chunkSize: 1000 }) {
        const insertRows = newEntityRows.map((newEntityRow) => this.columnsForCreate
            ? this.pickWithoutUndefined(newEntityRow, this.columnsForCreate, this.columnsForCreateSet)
            : (0, knex_utils_1.copyWithoutUndefined)(newEntityRow));
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const chunks = (0, knex_utils_1.chunk)(insertRows, params.chunkSize);
        for (const rows of chunks) {
            await queryBuilder(this.tableName).insert(rows);
        }
    }
    async updateById(id, updatedFields, transactionProvider, updateConfig = {}) {
        const updatedColumns = this.columnsForUpdate
            ? this.pickWithoutUndefined(updatedFields, this.columnsForUpdate, this.columnsForUpdateSet)
            : (0, knex_utils_1.copyWithoutUndefined)(updatedFields);
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const qb = queryBuilder(this.tableName)
            .where({ [this.idColumn]: id })
            .update(updatedColumns)
            .returning(this.columnsToFetch);
        if (updateConfig.timeout) {
            qb.timeout(updateConfig.timeout);
        }
        const updatedUserRows = await qb;
        return updatedUserRows[0];
    }
    async updateSingleByCriteria(filterCriteria, updatedFields, transactionProvider) {
        const result = await this.updateByCriteria(filterCriteria, updatedFields, transactionProvider);
        let returningResult;
        if ((0, dbSupportHelper_1.doesSupportReturning)(this.knex)) {
            returningResult = result;
        }
        else {
            returningResult = await this.getByCriteria(filterCriteria, transactionProvider, {
                columnsToFetch: this.columnsToFetchDetails,
            });
        }
        if (returningResult.length > 1) {
            throw new NonUniqueResultError_1.NonUniqueResultError('Query updated more than one row', filterCriteria);
        }
        if (returningResult.length === 0) {
            throw new NoEntityExistsError_1.NoEntityExistsError('Query updated no rows', filterCriteria);
        }
        return result[0];
    }
    async updateByCriteria(filterCriteria, updatedFields, transactionProvider, updateParams = {}) {
        var _a;
        const updatedColumns = this.columnsForUpdate
            ? this.pickWithoutUndefined(updatedFields, this.columnsForUpdate, this.columnsForUpdateSet)
            : (0, knex_utils_1.copyWithoutUndefined)(updatedFields);
        const filters = this.columnsForFilters
            ? this.pickWithoutUndefined(filterCriteria, this.columnsForFilters, this.columnsForFiltersSet)
            : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const qb = queryBuilder(this.tableName)
            .where(filters)
            .update(updatedColumns)
            .returning(this.columnsToFetch);
        const sortParam = (_a = updateParams.sorting) !== null && _a !== void 0 ? _a : this.defaultOrderBy;
        if ((0, dbSupportHelper_1.doesSupportUpdateOrderBy)(this.knex) && sortParam) {
            qb.orderBy(sortParam);
        }
        const result = await qb;
        return result;
    }
    async getById(id, transactionProvider, params = {}) {
        var _a;
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const result = await queryBuilder(this.tableName)
            .where({ [this.idColumn]: id })
            .select((_a = params.columnsToFetch) !== null && _a !== void 0 ? _a : this.columnsToFetchDetails);
        return result === null || result === void 0 ? void 0 : result[0];
    }
    async getByIdForUpdate(id, transactionProvider, params = {}) {
        var _a;
        const trx = await transactionProvider();
        const result = await this.knex(this.tableName)
            .transacting(trx)
            .forUpdate()
            .where({ [this.idColumn]: id })
            .select((_a = params.columnsToFetch) !== null && _a !== void 0 ? _a : this.columnsToFetchDetails);
        return result === null || result === void 0 ? void 0 : result[0];
    }
    async getByCriteria(filterCriteria, transactionProvider, params = {}) {
        var _a, _b;
        let filters;
        if (filterCriteria) {
            filters = this.columnsForFilters
                ? this.pickWithoutUndefined(filterCriteria, this.columnsForFilters, this.columnsForFiltersSet)
                : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        }
        else {
            filters = {};
        }
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const qb = queryBuilder(this.tableName)
            .select((_a = params.columnsToFetch) !== null && _a !== void 0 ? _a : this.columnsToFetchList)
            .where(filters);
        const sortParam = (_b = params.sorting) !== null && _b !== void 0 ? _b : this.defaultOrderBy;
        if (sortParam) {
            qb.orderBy(sortParam);
        }
        const result = await qb;
        return result;
    }
    async getByCriteriaForUpdate(transactionProvider, filterCriteria, params = {}) {
        var _a, _b;
        let filters;
        if (filterCriteria) {
            filters = this.columnsForFilters
                ? this.pickWithoutUndefined(filterCriteria, this.columnsForFilters, this.columnsForFiltersSet)
                : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        }
        else {
            filters = {};
        }
        const trx = await transactionProvider();
        const qb = this.knex(this.tableName)
            .transacting(trx)
            .forUpdate()
            .select((_a = params.columnsToFetch) !== null && _a !== void 0 ? _a : this.columnsToFetchList)
            .where(filters);
        const sortParam = (_b = params.sorting) !== null && _b !== void 0 ? _b : this.defaultOrderBy;
        if (sortParam) {
            qb.orderBy(sortParam);
        }
        const result = await qb;
        return result;
    }
    async getSingleByCriteria(filterCriteria, params = {}) {
        var _a;
        const result = await this.getByCriteria(filterCriteria, null, {
            columnsToFetch: (_a = params.columnsToFetch) !== null && _a !== void 0 ? _a : this.columnsToFetchDetails,
            sorting: params.sorting,
        });
        if (result.length > 1) {
            throw new NonUniqueResultError_1.NonUniqueResultError('Query resulted more than in a single result', filterCriteria);
        }
        return result[0];
    }
    async deleteById(id, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        await queryBuilder(this.tableName)
            .where({
            [this.idColumn]: id,
        })
            .del();
    }
    async deleteByCriteria(filterCriteria, transactionProvider) {
        const filters = this.columnsForFilters
            ? this.pickWithoutUndefined(filterCriteria, this.columnsForFilters, this.columnsForFiltersSet)
            : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        await queryBuilder(this.tableName).where(filters).del();
    }
    createTransactionProvider() {
        return this.knex.transactionProvider();
    }
    async commitTransaction(transactionProvider) {
        const trx = await transactionProvider();
        await trx.commit();
    }
    async rollbackTransaction(transactionProvider) {
        const trx = await transactionProvider();
        await trx.rollback();
    }
    async getKnexOrTransaction(transactionProvider) {
        return transactionProvider ? await transactionProvider() : this.knex;
    }
}
exports.AbstractRepository = AbstractRepository;
//# sourceMappingURL=AbstractRepository.js.map